/* $ Id: friends-postgres.sql,v 1.2.2.11 2004/06/03 11:15:43 pnixon Exp $ */
-- Postgres schema for centralised "friends"
-- Peter Nixon [ codemonkey@peternixon.net ]

-- Description: This system allows centralisation of all authentication into one
-- "friends" table. It includes VIEWs for each different channel type so
-- that this system can be used directly with Asterisk with no code modification
-- (Currently ast_data does need to be patched in, but this is expected to me in
-- mainline CVS before too long).
-- I use Postgres RULEs to make sure UPDATEs to the VIEWs get to the correct table
-- I have also added some extra columns to support app_prepaid directly from the
-- same table with no fuss nor muss.
-- Each channel and calling card can be independently disabled.
-- In future I plan on using these RULEs to also keep and audit trail of account
-- changes, but that will have to wait until another day to be implimented.
-- If anyone has any extra features they can think of, I would be happy to impliment
-- them also.

CREATE TABLE sipacl (
	id		BIGSERIAL PRIMARY KEY,	
	name		VARCHAR(40) NOT NULL default '',
	priority	INTEGER NOT NULL default '1',
	sense		INTEGER NOT NULL default 0,
	ipaddr		VARCHAR(20) NOT NULL default '',
	netmask		VARCHAR(20) NOT NULL default ''
);
CREATE UNIQUE INDEX sipacl_unique_index ON sipacl (name, priority);

CREATE TABLE friends (
	id			BIGSERIAL PRIMARY KEY,	
	name			VARCHAR(40) NOT NULL default '',
	secret			VARCHAR(40) NOT NULL default '',
	context			VARCHAR(40) NOT NULL default '',
	username		VARCHAR(40) default '',
	accountcode		VARCHAR(20) NOT NULL default '',
	mailbox			VARCHAR(80) default '',
	amaflags		INTEGER NOT NULL default '0',
	restrictcid		INTEGER NOT NULL default '0',
	type			INTEGER NOT NULL default '3',
	sip_ipaddr		INET NOT NULL default '0.0.0.0',
	sip_port		INTEGER NOT NULL default '0',
	sip_regseconds		BIGINT NOT NULL default '0',
	sip_nat			INTEGER NOT NULL default '3',
	sip_canreinvite		INTEGER NOT NULL default '0',
	sip_dtmfmode		INTEGER NOT NULL default '1',
	sip_cidname		VARCHAR(20) NOT NULL default '',
	sip_cidnumber		VARCHAR(20) NOT NULL default '',
	sip_codecflags		INTEGER NOT NULL default '524302',
	sip_useragent		VARCHAR(40) NOT NULL default '',
	iax2_ipaddr		INET NOT NULL default '0.0.0.0',
	iax2_port		INTEGER NOT NULL default '0',
	iax2_regseconds		BIGINT NOT NULL default '0',
	credit			INTEGER NOT NULL default '0',
	tariff			TEXT NOT NULL,
	tariffcode		TEXT NOT NULL,
	fullname		VARCHAR(80) NOT NULL DEFAULT '',
	email			VARCHAR(80) NOT NULL DEFAULT '',
	pager			VARCHAR(80) NOT NULL DEFAULT '',
	vm_password		VARCHAR(80) NOT NULL DEFAULT '',
	vm_options		VARCHAR(160) NOT NULL DEFAULT '',
	sip_activated		BOOLEAN NOT NULL DEFAULT 'true',
	iax2_activated		BOOLEAN NOT NULL DEFAULT 'true',
	vm_activated		BOOLEAN NOT NULL DEFAULT 'true',
	card_activated		BOOLEAN NOT NULL DEFAULT 'false',
	account_activated	BOOLEAN NOT NULL DEFAULT 'true'
);
CREATE UNIQUE INDEX friends_unique_index ON friends (name);


-- View and Rule for SIP Friends
CREATE OR REPLACE VIEW sipfriends AS
SELECT name, secret, context, username, sip_ipaddr AS ipaddr, sip_port AS port, sip_regseconds AS regseconds, type, mailbox, sip_nat AS nat, sip_canreinvite AS canreinvite, accountcode, amaflags, sip_dtmfmode AS dtmfmode, restrictcid, sip_cidnumber AS cidnumber, sip_codecflags AS codecflags, sip_useragent AS useragent
FROM friends
WHERE account_activated = true AND sip_activated = true;

CREATE OR REPLACE RULE sipfriends_ins_protect AS ON INSERT TO sipfriends
    DO INSTEAD NOTHING;
CREATE OR REPLACE RULE sipfriends_del_protect AS ON DELETE TO sipfriends
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE sipfriends_upd AS ON UPDATE TO sipfriends
    DO INSTEAD
    UPDATE friends
       SET sip_ipaddr = NEW.ipaddr,
           sip_port = NEW.port,
           sip_regseconds = NEW.regseconds,
           username = NEW.username,
           sip_useragent = NEW.useragent
     WHERE name = OLD.name;


-- View and Rule(s) for IAX Friends
CREATE OR REPLACE VIEW iaxfriends AS
SELECT name, secret, context, type, iax2_ipaddr AS ipaddr, iax2_port AS port, iax2_regseconds AS regseconds, accountcode
FROM friends
WHERE account_activated = true AND iax2_activated = true;

CREATE OR REPLACE RULE iaxfriends_ins_protect AS ON INSERT TO iaxfriends
    DO INSTEAD NOTHING;
CREATE OR REPLACE RULE iaxfriends_del_protect AS ON DELETE TO iaxfriends
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE iaxfriends_upd AS ON UPDATE TO iaxfriends
    DO INSTEAD
    UPDATE friends
       SET iax2_ipaddr = NEW.ipaddr,
           iax2_port = NEW.port,
           iax2_regseconds = NEW.regseconds
     WHERE name = OLD.name;


-- View and Rule(s) to allow integration of app_prepaid with friends
CREATE OR REPLACE VIEW card AS
SELECT name AS username, secret AS userpass, credit, tariff, card_activated AS activated
FROM friends;

CREATE OR REPLACE RULE card_ins_protect AS ON INSERT TO card
    DO INSTEAD NOTHING;
CREATE OR REPLACE RULE card_del_protect AS ON DELETE TO card
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE card_upd AS ON UPDATE TO card
    DO INSTEAD
    UPDATE friends
       SET credit = NEW.credit
     WHERE name = OLD.username;
