--
-- PostgreSQL database dump
--

SET client_encoding = 'UNICODE';
SET check_function_bodies = false;

SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 2 (OID 0)
-- Name: prepaid; Type: DATABASE; Schema: -; Owner: postgres
--

CREATE DATABASE prepaid WITH TEMPLATE = template0 ENCODING = 'UNICODE';


\connect prepaid postgres

SET client_encoding = 'UNICODE';
SET check_function_bodies = false;

SET search_path = public, pg_catalog;

--
-- TOC entry 35 (OID 17142)
-- Name: plpgsql_call_handler(); Type: FUNC PROCEDURAL LANGUAGE; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


SET SESSION AUTHORIZATION DEFAULT;

--
-- TOC entry 33 (OID 17143)
-- Name: plpgsql; Type: PROCEDURAL LANGUAGE; Schema: public; Owner: 
--

CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler;


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 5 (OID 17191)
-- Name: country; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE country (
    countrycode text NOT NULL,
    countryname text NOT NULL
);


--
-- TOC entry 6 (OID 17241)
-- Name: countryprefix; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE countryprefix (
    countryprefixid serial NOT NULL,
    prefix text NOT NULL,
    countrycode text NOT NULL,
    subcode text NOT NULL
);


--
-- TOC entry 37 (OID 28087)
-- Name: asterisk_authenticate(character varying); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION asterisk_authenticate(character varying) RETURNS SETOF record
    AS '
DECLARE

    p_cid ALIAS FOR $1;
    rec RECORD;

BEGIN

SELECT into rec
    credit::integer,
    tariffid::integer,
    activated::bool,
    asterisk_expirecheck(expires),
    cardid::text
FROM cid left join card using(cidid) left join cardtype using(cardtypeid) where cid=p_cid and payed;

IF FOUND THEN
    RETURN NEXT rec;
END IF;

RETURN;

END






'
    LANGUAGE plpgsql;


--
-- TOC entry 7 (OID 28091)
-- Name: cardtype; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE cardtype (
    cardtypeid serial NOT NULL,
    value integer DEFAULT 0 NOT NULL,
    description text NOT NULL,
    active boolean DEFAULT false NOT NULL,
    tariffid integer NOT NULL,
    lifetime integer DEFAULT 0 NOT NULL,
    frequency integer DEFAULT 30 NOT NULL,
    startpulse integer DEFAULT 30 NOT NULL
);


--
-- TOC entry 8 (OID 28104)
-- Name: cid; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE cid (
    cidid serial NOT NULL,
    cid text NOT NULL,
    resellerid integer NOT NULL,
    createtime timestamp without time zone NOT NULL
);


--
-- TOC entry 9 (OID 28110)
-- Name: card; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE card (
    cardid text NOT NULL,
    cidid integer,
    payed boolean DEFAULT false NOT NULL,
    cardtypeid integer NOT NULL,
    resellerid integer NOT NULL,
    activated boolean DEFAULT false NOT NULL,
    credit integer DEFAULT 0 NOT NULL,
    deactivated timestamp without time zone,
    expires timestamp without time zone
);


--
-- TOC entry 10 (OID 28120)
-- Name: reseller; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE reseller (
    resellerid serial NOT NULL,
    username text NOT NULL,
    userpass text NOT NULL
);


--
-- TOC entry 11 (OID 28150)
-- Name: sale; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE sale (
    saleid serial NOT NULL,
    cardid integer NOT NULL,
    saletime timestamp without time zone NOT NULL
);


--
-- TOC entry 12 (OID 28161)
-- Name: order; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE "order" (
    orderid serial NOT NULL,
    cardid integer NOT NULL,
    ordertime timestamp without time zone NOT NULL
);


--
-- TOC entry 13 (OID 28172)
-- Name: provider; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE provider (
    providerid serial NOT NULL,
    carriercode text NOT NULL,
    providertech text NOT NULL,
    providerip text,
    providername text NOT NULL
);


--
-- TOC entry 14 (OID 28182)
-- Name: providerdestination; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE providerdestination (
    providerdestinationid serial NOT NULL,
    countryprefixid integer NOT NULL,
    providerid integer NOT NULL
);


--
-- TOC entry 15 (OID 28193)
-- Name: providerrate; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE providerrate (
    providerrateid serial NOT NULL,
    providerid integer NOT NULL,
    countryprefixid integer NOT NULL,
    rate real DEFAULT 0 NOT NULL
);


--
-- TOC entry 16 (OID 28213)
-- Name: tariffrate; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE tariffrate (
    tariffrateid serial NOT NULL,
    tariffid integer NOT NULL,
    countryprefixid integer NOT NULL,
    rate real DEFAULT 0 NOT NULL
);


--
-- TOC entry 17 (OID 28225)
-- Name: tariff; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE tariff (
    tariffid serial NOT NULL,
    tariffname text NOT NULL
);


--
-- TOC entry 18 (OID 28245)
-- Name: call; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE call (
    callid serial NOT NULL,
    providerid integer,
    countryprefixid integer,
    uniqueid text NOT NULL,
    tariffrateid integer NOT NULL,
    channel text NOT NULL,
    starttime timestamp without time zone NOT NULL,
    stoptime timestamp without time zone,
    startdelay integer DEFAULT 0 NOT NULL,
    stopdelay integer DEFAULT 0 NOT NULL,
    sessiontime integer DEFAULT 0 NOT NULL,
    callednumber text NOT NULL,
    terminatecause text,
    sessionbill integer DEFAULT 0 NOT NULL,
    pulsetime integer DEFAULT 0,
    cardid text NOT NULL
);


--
-- TOC entry 36 (OID 28283)
-- Name: asterisk_expirecheck(timestamp without time zone); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION asterisk_expirecheck(timestamp without time zone) RETURNS boolean
    AS '
DECLARE
  expire ALIAS FOR $1;

BEGIN
  IF ((expire is NULL) OR (expire>now())) THEN
    RETURN false;
  ELSE
    RETURN true;
  END IF;
END;




'
    LANGUAGE plpgsql;


--
-- TOC entry 38 (OID 28298)
-- Name: asterisk_acct_start(character varying, character varying, character varying, timestamp without time zone, integer, integer, integer, character varying); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION asterisk_acct_start(character varying, character varying, character varying, timestamp without time zone, integer, integer, integer, character varying) RETURNS integer
    AS '
DECLARE
    p_uniqueid ALIAS FOR $1;
    p_channel ALIAS FOR $2;
    p_cardid ALIAS FOR $3;
    p_starttime ALIAS FOR $4;
    p_providerid ALIAS FOR $5;
    p_countryprefixid ALIAS FOR $6;
    p_tariffrateid ALIAS FOR $7;
    p_callednumber ALIAS FOR $8;

BEGIN

INSERT INTO call (uniqueid,channel,cardid,starttime,providerid,countryprefixid,tariffrateid,callednumber) VALUES (p_uniqueid, p_channel, p_cardid, p_starttime, p_providerid, p_countryprefixid,p_tariffrateid, p_callednumber);

RETURN 0;

END


'
    LANGUAGE plpgsql;


--
-- TOC entry 39 (OID 28309)
-- Name: asterisk_acct_stop(character varying, integer, character varying, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION asterisk_acct_stop(character varying, integer, character varying, timestamp without time zone) RETURNS integer
    AS '
DECLARE
    p_uniqueid ALIAS FOR $1;
    p_sessiontime ALIAS FOR $2;
    p_terminatecause ALIAS FOR $3; 
    p_stoptime ALIAS FOR $4;
    
BEGIN


UPDATE call SET sessiontime=p_sessiontime, terminatecause=p_terminatecause, stoptime=p_stoptime WHERE uniqueid=p_uniqueid;

RETURN 0;

END

'
    LANGUAGE plpgsql;


--
-- TOC entry 34 (OID 28314)
-- Name: asterisk_authorize(integer, character varying); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION asterisk_authorize(integer, character varying) RETURNS SETOF record
    AS '
DECLARE
    p_tariffid ALIAS FOR $1;
    p_destination ALIAS FOR $2;
    
    l_rate INTEGER;
    l_countrycode TEXT;
    l_subcode TEXT;
    l_prefix TEXT;
    l_countryprefixid INTEGER;
    l_tariffrateid INTEGER;
    rec RECORD;
    recfirst RECORD;

BEGIN
SELECT into recfirst rate, prefix, countrycode, subcode, countryprefixid, tariffrateid FROM countryprefix LEFT JOIN tariffrate USING (countryprefixid) WHERE tariffid=p_tariffid AND prefix=substring(p_destination,3,length(prefix)) ORDER BY LENGTH(prefix) DESC LIMIT 1;
l_rate := recfirst.rate;
l_prefix := recfirst.prefix;
l_countrycode := recfirst.countrycode;
l_subcode := recfirst.subcode;
l_countryprefixid := recfirst.countryprefixid;
l_tariffrateid := recfirst.tariffrateid;

SELECT into rec
    l_rate::integer,
    l_tariffrateid,
    carriercode::text,
    providertech::text,
    providerip::text,
    providerid::integer,
    countryprefixid::integer
 from providerrate left join provider using(providerid) where countryprefixid=l_countryprefixid;

IF FOUND THEN
    RETURN NEXT rec;
END IF;

RETURN;

END





'
    LANGUAGE plpgsql;


--
-- TOC entry 20 (OID 17247)
-- Name: countryprefix_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY countryprefix
    ADD CONSTRAINT countryprefix_pkey PRIMARY KEY (countryprefixid);


--
-- TOC entry 23 (OID 28126)
-- Name: cardid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY card
    ADD CONSTRAINT cardid_pkey PRIMARY KEY (cardid);


--
-- TOC entry 22 (OID 28128)
-- Name: cidid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY cid
    ADD CONSTRAINT cidid_pkey PRIMARY KEY (cidid);


--
-- TOC entry 21 (OID 28134)
-- Name: cardtypeid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY cardtype
    ADD CONSTRAINT cardtypeid_pkey PRIMARY KEY (cardtypeid);


--
-- TOC entry 24 (OID 28140)
-- Name: resellerid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY reseller
    ADD CONSTRAINT resellerid_pkey PRIMARY KEY (resellerid);


--
-- TOC entry 25 (OID 28153)
-- Name: saleid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY sale
    ADD CONSTRAINT saleid_pkey PRIMARY KEY (saleid);


--
-- TOC entry 26 (OID 28164)
-- Name: orderid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY "order"
    ADD CONSTRAINT orderid_pkey PRIMARY KEY (orderid);


--
-- TOC entry 27 (OID 28178)
-- Name: providerid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY provider
    ADD CONSTRAINT providerid_pkey PRIMARY KEY (providerid);


--
-- TOC entry 28 (OID 28185)
-- Name: providerdestinationid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY providerdestination
    ADD CONSTRAINT providerdestinationid_pkey PRIMARY KEY (providerdestinationid);


--
-- TOC entry 29 (OID 28197)
-- Name: providerrateid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY providerrate
    ADD CONSTRAINT providerrateid_pkey PRIMARY KEY (providerrateid);


--
-- TOC entry 30 (OID 28217)
-- Name: tariffrateid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY tariffrate
    ADD CONSTRAINT tariffrateid_pkey PRIMARY KEY (tariffrateid);


--
-- TOC entry 31 (OID 28231)
-- Name: tariffid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY tariff
    ADD CONSTRAINT tariffid_pkey PRIMARY KEY (tariffid);


--
-- TOC entry 19 (OID 28237)
-- Name: countrycode_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY country
    ADD CONSTRAINT countrycode_pkey PRIMARY KEY (countrycode);


--
-- TOC entry 32 (OID 28253)
-- Name: callid_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY call
    ADD CONSTRAINT callid_pkey PRIMARY KEY (callid);


--
-- TOC entry 42 (OID 28130)
-- Name: cidid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY card
    ADD CONSTRAINT cidid_fkey FOREIGN KEY (cidid) REFERENCES cid(cidid);


--
-- TOC entry 43 (OID 28136)
-- Name: cardtypeid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY card
    ADD CONSTRAINT cardtypeid_fkey FOREIGN KEY (cardtypeid) REFERENCES cardtype(cardtypeid);


--
-- TOC entry 44 (OID 28142)
-- Name: resellerid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY card
    ADD CONSTRAINT resellerid_fkey FOREIGN KEY (resellerid) REFERENCES reseller(resellerid);


--
-- TOC entry 45 (OID 28155)
-- Name: cardid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY sale
    ADD CONSTRAINT cardid_fkey FOREIGN KEY (cardid) REFERENCES card(cardid);


--
-- TOC entry 46 (OID 28166)
-- Name: cardid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY "order"
    ADD CONSTRAINT cardid_fkey FOREIGN KEY (cardid) REFERENCES card(cardid);


--
-- TOC entry 47 (OID 28187)
-- Name: providerid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY providerdestination
    ADD CONSTRAINT providerid_fkey FOREIGN KEY (providerid) REFERENCES provider(providerid);


--
-- TOC entry 49 (OID 28199)
-- Name: providerid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY providerrate
    ADD CONSTRAINT providerid_fkey FOREIGN KEY (providerid) REFERENCES provider(providerid);


--
-- TOC entry 50 (OID 28203)
-- Name: countryprefixid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY providerrate
    ADD CONSTRAINT countryprefixid_fkey FOREIGN KEY (countryprefixid) REFERENCES countryprefix(countryprefixid);


--
-- TOC entry 48 (OID 28207)
-- Name: countryprefixid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY providerdestination
    ADD CONSTRAINT countryprefixid_fkey FOREIGN KEY (countryprefixid) REFERENCES countryprefix(countryprefixid);


--
-- TOC entry 51 (OID 28219)
-- Name: countryprefixid; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY tariffrate
    ADD CONSTRAINT countryprefixid FOREIGN KEY (countryprefixid) REFERENCES countryprefix(countryprefixid);


--
-- TOC entry 52 (OID 28233)
-- Name: tariffid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY tariffrate
    ADD CONSTRAINT tariffid_fkey FOREIGN KEY (tariffid) REFERENCES tariff(tariffid);


--
-- TOC entry 40 (OID 28239)
-- Name: countrycode_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY countryprefix
    ADD CONSTRAINT countrycode_fkey FOREIGN KEY (countrycode) REFERENCES country(countrycode);


--
-- TOC entry 53 (OID 28255)
-- Name: providerid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY call
    ADD CONSTRAINT providerid_fkey FOREIGN KEY (providerid) REFERENCES provider(providerid);


--
-- TOC entry 54 (OID 28263)
-- Name: countryprefixid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY call
    ADD CONSTRAINT countryprefixid_fkey FOREIGN KEY (countryprefixid) REFERENCES countryprefix(countryprefixid);


--
-- TOC entry 55 (OID 28267)
-- Name: tarifrateid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY call
    ADD CONSTRAINT tarifrateid_fkey FOREIGN KEY (tariffrateid) REFERENCES tariffrate(tariffrateid);


--
-- TOC entry 41 (OID 28273)
-- Name: resellerid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY cid
    ADD CONSTRAINT resellerid_fkey FOREIGN KEY (resellerid) REFERENCES reseller(resellerid);


--
-- TOC entry 56 (OID 28284)
-- Name: cardid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY call
    ADD CONSTRAINT cardid_fkey FOREIGN KEY (cardid) REFERENCES card(cardid);


--
-- TOC entry 3 (OID 2200)
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--

COMMENT ON SCHEMA public IS 'Standard public schema';


