-- $Id: prepaid-functions-asterisk.sql,v 1.7 2004/08/23 13:34:49 tuyano Exp $

CREATE OR REPLACE FUNCTION asterisk_authenticate_cid(character varying) RETURNS SETOF record
    AS '
DECLARE

    p_cid ALIAS FOR $1;
    rec RECORD;

BEGIN

SELECT into rec
    username::text,
    activated::bool
FROM callerid where cid=p_cid;

IF FOUND THEN
    RETURN NEXT rec;
END IF;

RETURN;

END

'
    LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION asterisk_authorize_user(character varying) RETURNS SETOF record
    AS '
DECLARE

    p_user ALIAS FOR $1;
    rec RECORD;

BEGIN

SELECT into rec
    tariffcode::text,
    account_activated::bool
FROM friends where UserName=p_user;

IF FOUND THEN
    RETURN NEXT rec;
END IF;

RETURN;

END

'
    LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION asterisk_authenticate(character varying, character varying) RETURNS SETOF record
    AS '

DECLARE

    p_username ALIAS FOR $1;
    p_userpass ALIAS FOR $2;
    rec RECORD;

BEGIN

SELECT into rec 
    credit::integer,
    tariff::text,
    activated::bool
FROM card 
WHERE username=p_username AND userpass=p_userpass;

IF FOUND THEN
    RETURN NEXT rec;
END IF;

RETURN;

END

'
    LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION asterisk_authorize(character varying, character varying) RETURNS SETOF record
    AS '

DECLARE

    p_tariffcode ALIAS FOR $1;
    p_destination ALIAS FOR $2;
    
    l_rate INTEGER;
    l_destination VARCHAR;
    rec RECORD;

BEGIN

SELECT into l_destination
SUBSTRING(
    p_destination,
    (SELECT length(prefix)+1 from internationalprefix WHERE p_destination LIKE prefix || ''%'' order by length(prefix) desc limit 1)
);


SELECT into l_rate
    rate::integer
FROM countryprefix LEFT JOIN tariffrate USING (countrycode,subcode)
WHERE tariffcode=p_tariffcode AND prefix=SUBSTRING(
    p_destination,
    (SELECT length(prefix)+1 from internationalprefix WHERE p_destination LIKE prefix || ''%'' order by length(prefix) desc limit 1),
    length(prefix)
)
ORDER BY LENGTH(prefix) DESC LIMIT 1;


SELECT into rec
    l_rate,
    providertech::text,
    prefix::text,
    providerip::text,
    l_destination::text
FROM providerdestination LEFT JOIN provider USING (providercode) 
WHERE destination=SUBSTRING(p_destination, 1, length(destination))
ORDER BY LENGTH(destination) DESC LIMIT 1;

IF FOUND THEN
    RETURN NEXT rec;
END IF;

RETURN;

END

'
    LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION asterisk_acct_start(character varying, character varying, character varying, character varying, timestamp) RETURNS INTEGER
    AS '

DECLARE

    p_uniqueid ALIAS FOR $1;
    p_sessionid ALIAS FOR $2;
    p_username ALIAS FOR $3;
    p_nasipaddress ALIAS FOR $4;
    p_starttime ALIAS FOR $5;
    
BEGIN

INSERT INTO call (uniqueid,sessionid,username,nasipaddress,starttime) VALUES (p_uniqueid, p_sessionid, p_username, p_nasipaddress, p_starttime);

RETURN 0;

END

'
    LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION asterisk_acct_stop(character varying, character varying, character varying, character varying, integer, integer, character varying, timestamp) RETURNS INTEGER
    AS '

DECLARE

    p_uniqueid ALIAS FOR $1;
    p_sessionid ALIAS FOR $2;
    p_username ALIAS FOR $3;
    p_calledstation ALIAS FOR $4;
    p_rate ALIAS FOR $5;
    p_sessiontime ALIAS FOR $6;
    p_terminatecause ALIAS FOR $7; 
    p_stoptime ALIAS FOR $8;

    used INTEGER;
    
BEGIN


UPDATE call SET sessiontime=p_sessiontime, calledstation=p_calledstation, terminatecause=p_terminatecause, stoptime=p_stoptime WHERE uniqueid=p_uniqueid;

used := round(p_rate * p_sessiontime / 60,0);

update card set credit=(credit - used) where username=p_username;

RETURN 0;

END

'
    LANGUAGE plpgsql;

